package edu.sjsu.rhinofaces;

import java.beans.FeatureDescriptor;
import java.util.Iterator;
import java.util.logging.Logger;

import javax.el.CompositeELResolver;
import javax.el.ELContext;
import javax.el.ELResolver;

/**
   This defers to the RhinoELResolver class for JSVar object.
   All others are delegated to the regular CompositeELResolver 
   @author Tom Austin
 */
public class RhinoCompositeResolver extends ELResolver
{
   private static Logger logger = Logger.global;
   
   private ELResolver delegate;
   private RhinoELResolver jsResolver;
   
   public RhinoCompositeResolver(ELResolver delegate, RhinoELResolver jsResolver)
   {
      this.delegate = delegate;
      this.jsResolver = jsResolver;
   }
   
   @Override
   public Object getValue(ELContext context, Object base, Object property)
   {
      logger.info("RhinoCompositeResolver: " + base + "." + property);
      if (base instanceof JSVar)
      {
         return this.jsResolver.getValue(context, base, property);
      }
      else return delegate.getValue(context, base, property);
   }
   
   @Override
   public void setValue(ELContext context, Object base, Object property, Object value)
   {
      if (base instanceof JSVar)
      {
         this.jsResolver.setValue(context, base, property, value);
      }
      else delegate.setValue(context, base, property, value);
   }

   /**
      @see javax.el.ELResolver#getCommonPropertyType(javax.el.ELContext, java.lang.Object)
    */
   @Override
   public Class<?> getCommonPropertyType(ELContext arg0, Object arg1)
   {
      return delegate.getCommonPropertyType(arg0, arg1);
   }

   /**
      @see javax.el.ELResolver#getFeatureDescriptors(javax.el.ELContext, java.lang.Object)
    */
   @Override
   public Iterator<FeatureDescriptor> getFeatureDescriptors(ELContext arg0, Object arg1)
   {
      return delegate.getFeatureDescriptors(arg0, arg1);
   }

   /**
      @see javax.el.ELResolver#getType(javax.el.ELContext, java.lang.Object, java.lang.Object)
    */
   @Override
   public Class<?> getType(ELContext arg0, Object arg1, Object arg2)
   {
      //Return null so that the application does not try to check for JavaScript functions
      if (arg1 instanceof JSVar) return null;
      else return delegate.getType(arg0, arg1, arg2);
   }

   /**
      @see javax.el.ELResolver#isReadOnly(javax.el.ELContext, java.lang.Object, java.lang.Object)
    */
   @Override
   public boolean isReadOnly(ELContext arg0, Object arg1, Object arg2)
   {
      return delegate.isReadOnly(arg0, arg1, arg2);
   }
}
